@@ -3,10 +3,10 @@
  *  Module    : makecfg.c
  *  Author    : Thomas E. Dickey
  *  Created   : 1997-08-23
- *  Updated   : 2022-04-09
+ *  Updated   : 2024-12-21
  *  Notes     : #defines and structs for options_menu.c
  *
- * Copyright (c) 1997-2023 Thomas E. Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2025 Thomas E. Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,12 +37,70 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+/* replace tin.h {{ */
 
-#define __BUILD__
-#define PROTO_H 1	/* don't include proto.h */
-#ifndef TIN_H
-#	include "tin.h"
-#endif /* !TIN_H */
+#ifdef HAVE_CONFIG_H
+#include "autoconf.h"
+#endif /* HAVE_CONFIG_H */
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif /* HAVE_STDDEF_H */
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif /* HAVE_CONFIG_H */
+
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#else
+extern void exit(int);
+extern void free(void *);
+#endif /* HAVE_STDLIB_H */
+
+#ifndef EXIT_SUCCESS
+#define EXIT_SUCCESS 0
+#endif /* !EXIT_SUCCESS */
+
+#ifndef EXIT_FAILURE
+#define EXIT_FAILURE 1
+#endif /* !EXIT_FAILURE */
+
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#else
+#if !defined(HAVE_STDLIB_H)
+extern void *malloc(size_t);
+#endif /* !HAVE_STDLIB_H */
+#endif /* HAVE_MALLOC_H */
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+extern char *strcpy(char *, const char *);
+extern int strcmp(const char *, const char *);
+extern int strncmp(const char *, const char *, size_t);
+extern size_t strlen(const char *);
+#endif /* HAVE_STRING_H */
+
+#include <stdio.h>
+#include <ctype.h>
+
+#define TRUE 1
+#define FALSE 0
+#define FreeIfNeeded(p)	if (p != NULL) free((void *) p)
+
+#ifdef STDC_NORETURN
+#ifdef HAVE_STDNORETURN_H
+#include <stdnoreturn.h>
+#endif /* HAVE_STDNORETURN_H */
+#else
+#if defined(_Noreturn)
+#undef _Noreturn
+#endif /* _Noreturn */
+#define _Noreturn /**/
+#endif /* STDC_NORETURN */
+
+/* }} replace tin.h */
 
 #define L_CURL '{'
 #define R_CURL '}'
@@ -60,7 +118,7 @@ MYDATA {
 static MYDATA *all_data;
 static int line_no;
 
-_Noreturn static void
+static _Noreturn void
 failed(
 	const char *message)
 {
@@ -100,7 +158,7 @@ store_data(
 	MYDATA *p, *q;
 
 	if ((p = (MYDATA *) malloc(sizeof(MYDATA))) == NULL)
-		failed("malloc() failed");
+		failed("malloc() failed ");
 
 	p->link = NULL;
 	p->name = string_dup(name);
@@ -125,12 +183,12 @@ parse_tbl(
 	/* strip leading/trailing blanks */
 	do {
 		t--;
-		if (isspace ((int)*t))
+		if (isspace((unsigned char) *t))
 			*t = '\0';
 		else
 			break;
 	} while (t > s);
-	while (isspace ((int)*s))
+	while (isspace((unsigned char) *s))
 		s++;
 	buffer = s;
 
@@ -143,9 +201,9 @@ parse_tbl(
 			 * otherwise the data consists of 2 blank
 			 * separated columns (name, type).
 			 */
-			while (*s && !isspace ((int)*s))
+			while (*s && !isspace((unsigned char) *s))
 				s++;
-			while (isspace ((int)*s))
+			while (isspace((unsigned char) *s))
 				*s++ = '\0';
 			store_data(buffer, s);
 		}
@@ -199,13 +257,14 @@ typename_of(
 	MYDATA *p)
 {
 	if (!strcmp(p->type, "OPT_STRING"))
-		return "char *";
-	if (!strcmp(p->type, "OPT_CHAR"))
+		return "char **";
+	if (!strcmp(p->type, "OPT_CHAR")) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		return "wchar_t *";
 #else
 		return "char *";
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	}
 	if (!strcmp(p->type, "OPT_ON_OFF"))
 		return "t_bool *";
 	return "int *";
@@ -226,12 +285,12 @@ generate_tbl(
 		,"#define OPT_CHAR      5"
 		,""
 		,"struct t_option option_table[]={"
-		,0
+		,NULL
 	};
 	static const char *const table_2[] =
 	{
 		"};"
-		,0
+		,NULL
 	};
 	static const char prefix[] = "  { ";
 	static const char suffix[] = "},";
@@ -261,7 +320,7 @@ generate_tbl(
 			if (is_title(p)) {
 				fprintf(ofp, "0, NULL, ");
 			} else if (!is_int) {
-				fprintf(ofp, "OINX_%s, 0, ", p->name);
+				fprintf(ofp, "OINX_%s, NULL, ", p->name);
 			} else {
 				fprintf(ofp, "0, &tinrc.%s, ", p->name);
 			}
@@ -286,13 +345,13 @@ generate_enum(
 
 	static const char *const table_1[] = {
 		"enum option_enum {",
-		0
+		NULL
 	};
 	static const char *const table_2[] = {
 		"\tSIGNAL_HANDLER };",
 		"",
 		"#define LAST_OPT (SIGNAL_HANDLER - 1)",
-		0
+		NULL
 	};
 
 	/* generate enumerated type */
@@ -326,7 +385,6 @@ generate_ptr(
 {
 	MYDATA *p, *q;
 	int after;
-	const char *addr = !strcmp(opt_type, "OPT_STRING") ? "" : "&";
 
 	switch (mode) {
 	case 0:
@@ -341,7 +399,8 @@ generate_ptr(
 	}
 	after = FALSE;
 
-	for (p = all_data, q = NULL; p != NULL; p = p->link) {
+	q = NULL;
+	for (p = all_data; p != NULL; p = p->link) {
 		if (p->name[0] == '#') {
 			if (!strcmp(p->name, "#endif")) {
 				if (after) {
@@ -360,10 +419,9 @@ generate_ptr(
 			}
 			switch (mode) {
 			case 0:
-				fprintf(ofp, "\t%stinrc.%s,%*s/* %2d: %s__ */\n",
-					addr,
+				fprintf(ofp, "\t&tinrc.%s,%*s/* %2d: %s__ */\n",
 					p->name,
-					MAXNAME - (int)(strlen(addr) + strlen(p->name)),
+					MAXNAME - (int) (1 + strlen(p->name)),
 					" ",
 					index_of(p),
 					p->name);
@@ -440,7 +498,7 @@ makecfg(
 		,"#\t\tdefine OTYP struct"
 		,"#\tendif /* CPP_DOES_CONCAT */"
 		,"#endif /* lint */"
-		,0
+		,NULL
 	};
 	static const char *const table_2[] =
 	{
@@ -452,7 +510,7 @@ makecfg(
 		,"#undef OTYP"
 		,""
 		,"#endif /* TINCFG_H */"
-		,0
+		,NULL
 	};
 	static const char *const table_3[] = {
 		"/* This file is generated by MAKECFG */",
@@ -460,12 +518,12 @@ makecfg(
 		"#ifndef OPTIONS_MENU_H",
 		"#define OPTIONS_MENU_H 1",
 		"",
-		0
+		NULL
 	};
 	static const char *const table_4[] = {
 		"",
 		"#endif /* OPTIONS_MENU_H */",
-		0
+		NULL
 	};
 
 	/*
