@@ -3,35 +3,38 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2018-11-22
+ *  Updated   : 2024-11-25
  *  Notes     :
  *
- * Copyright (c) 1991-2019 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote
- *    products derived from this software without specific prior written
- *    permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * 3. Neither the name of the copyright holder nor the names of its
+ *    contributors may be used to endorse or promote products derived from
+ *    this software without specific prior written permission.
+ *
+ * * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 
@@ -60,15 +63,14 @@ static void catchup_group(struct t_group *group, t_bool goto_next_unread_group);
 static void draw_group_arrow(void);
 static void read_groups(void);
 static void select_done(void);
-static void select_quit(void);
+_Noreturn static void select_quit(void);
 static void select_read_group(void);
 static void sort_active_file(void);
 static void subscribe_pattern(const char *prompt, const char *message, const char *result, t_bool state);
 static void sync_active_file(void);
 static void yank_active_file(void);
 #ifdef NNTP_ABLE
-	static char *lookup_msgid(char *msgid);
-	static int show_article_by_msgid(void);
+	static char *lookup_msgid(const char *msgid);
 	static struct t_group *get_group_from_list(char *newsgroups);
 #endif /* NNTP_ABLE */
 
@@ -102,7 +104,7 @@ select_right(
 }
 
 
-void
+_Noreturn void
 selection_page(
 	int start_groupnum,
 	int num_cmd_line_groups)
@@ -111,6 +113,9 @@ selection_page(
 	char key[MAXKEYLEN];
 	int i, n;
 	t_function func;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wtmp;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	selmenu.curr = start_groupnum;
 
@@ -121,6 +126,20 @@ selection_page(
 	Raw(TRUE);
 	ClearScreen();
 
+#ifdef NNTP_ABLE
+	if (cmdline.msgid) { /* -L cmd */
+		switch (show_article_by_msgid(cmdline.msgid)) {
+			case LOOKUP_ART_UNAVAIL:
+				wait_message(2, _(txt_art_unavailable));
+				break;
+
+			default:
+				break;
+		}
+		FreeAndNull(cmdline.msgid);
+	}
+#endif /* NNTP_ABLE */
+
 	/*
 	 * If user specified only 1 cmd line groupname (eg. tin -r alt.sources)
 	 * then go there immediately.
@@ -129,6 +148,7 @@ selection_page(
 		select_read_group();
 
 	cursoroff();
+
 	show_selection_page();	/* display group selection page */
 
 	forever {
@@ -137,7 +157,7 @@ selection_page(
 				show_selection_page();
 		} else {
 			if (!yanked_out)
-				yanked_out = bool_not(yanked_out); /* yank out if yanked in */
+				yanked_out = TRUE; /* yank out if yanked in */
 		}
 
 		set_xclick_on();
@@ -261,34 +281,36 @@ selection_page(
 				break;
 
 			case GLOBAL_EDIT_FILTER:
-				if (invoke_editor(filter_file, filter_file_offset, NULL))
-					(void) read_filter_file(filter_file);
-				show_selection_page();
+				if (no_write)
+					info_message(_(txt_info_no_write));
+				else {
+					if (invoke_editor(filter_file, filter_file_offset, NULL))
+						(void) read_filter_file(filter_file);
+					show_selection_page();
+				}
 				break;
 
 			case SELECT_TOGGLE_DESCRIPTIONS:	/* toggle newsgroup descriptions */
 				if (sel_fmt.show_grpdesc) {
-					show_description = bool_not(show_description);
-					if (show_description)
+					if ((show_description = bool_not(show_description)))
 						read_descriptions(TRUE);
+					need_parse_fmt |= SELECT_LEVEL;
 					show_selection_page();
 				} else
 					info_message(_(txt_grpdesc_disabled));
 				break;
 
 			case SELECT_GOTO:			/* prompt for a new group name */
-				{
-					int oldmax = selmenu.max;
+				i = selmenu.max;
 
-					if ((n = choose_new_group()) >= 0) {
-						/*
-						 * If a new group was added and it is on the actual screen
-						 * draw it. If it is off screen the redraw will handle it.
-						 */
-						if (oldmax != selmenu.max && n >= selmenu.first && n < selmenu.first + NOTESLINES)
-							build_gline(n);
-						move_to_item(n);
-					}
+				if ((n = choose_new_group()) >= 0) {
+					/*
+					 * If a new group was added and it is on the actual screen
+					 * draw it. If it is off screen the redraw will handle it.
+					 */
+					if (i != selmenu.max && n >= selmenu.first && n < selmenu.first + NOTESLINES)
+						build_gline(n);
+					move_to_item(n);
 				}
 				break;
 
@@ -297,14 +319,23 @@ selection_page(
 				show_selection_page();
 				break;
 
+			case GLOBAL_CONNECTION_INFO:
+				show_connection_page();
+				show_selection_page();
+				break;
+
 #ifdef NNTP_ABLE
 			case GLOBAL_LOOKUP_MESSAGEID:
-				switch (n = show_article_by_msgid()) {
-					case 0:
+				switch (show_article_by_msgid(NULL)) {
+					case LOOKUP_OK:
 						show_selection_page();
 						break;
 
-					case GRP_QUIT:
+					case LOOKUP_UNAVAIL:
+						info_message("%s %s", _(txt_lookup_func_not_available), _(txt_lookup_func_not_nntp));
+						break;
+
+					case LOOKUP_QUIT:
 						select_quit();
 						break;
 
@@ -321,6 +352,7 @@ selection_page(
 
 			case GLOBAL_TOGGLE_INVERSE_VIDEO:
 				toggle_inverse_video();
+				need_parse_fmt |= SELECT_LEVEL;
 				show_selection_page();
 				show_inverse_video_status();
 				break;
@@ -362,12 +394,7 @@ selection_page(
 				if (selmenu.curr < selmenu.first || selmenu.curr >= selmenu.first + NOTESLINES - 1 || selmenu.curr != n)
 					show_selection_page();
 				else {
-					i = selmenu.curr;
-					selmenu.curr = n;
-					erase_arrow();
-					selmenu.curr = i;
-					clear_message();
-					draw_group_arrow();
+					HpGlitch(draw_group_arrow());
 				}
 				break;
 
@@ -389,8 +416,10 @@ selection_page(
 				break;
 
 			case SELECT_QUIT_NO_WRITE:		/* quit, but don't save configuration */
-				if (prompt_yn(_(txt_quit_no_write), TRUE) == 1)
+				if (prompt_yn(_(txt_quit_no_write), TRUE) == 1) {
+					FreeAndNull(sel_fmt.str);
 					tin_done(EXIT_SUCCESS, NULL);
+				}
 				show_selection_page();
 				break;
 
@@ -405,7 +434,7 @@ selection_page(
 				 * as we effectively do a yank out on each change, set yanked_out accordingly
 				 */
 				yanked_out = TRUE;
-				wait_message(0, _(txt_reading_groups), (tinrc.show_only_unread_groups) ? _("unread") : _("all"));
+				wait_message(0, _(txt_reading_groups), (tinrc.show_only_unread_groups) ? _(txt_unread) : _(txt_all));
 
 				toggle_my_groups(NULL);
 				show_selection_page();
@@ -454,7 +483,11 @@ selection_page(
 					break;
 				}
 				if (CURR_GROUP.subscribed) {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+					mark_screen(selmenu.curr, flags_offset, CURR_GROUP.newgroup ? (const wchar_t *) L"N" : (const wchar_t *) L"u");
+#else
 					mark_screen(selmenu.curr, flags_offset, CURR_GROUP.newgroup ? "N" : "u");
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 					subscribe(&CURR_GROUP, UNSUBSCRIBED, TRUE);
 					info_message(_(txt_unsubscribed_to), CURR_GROUP.name);
 					move_down();
@@ -490,18 +523,21 @@ selection_page(
 						info_message(_(txt_cannot_post));
 						break;
 					}
-					snprintf(buf, sizeof(buf), _(txt_post_newsgroups), tinrc.default_post_newsgroups);
-					if (!prompt_string_default(buf, tinrc.default_post_newsgroups, _(txt_no_newsgroups), HIST_POST_NEWSGROUPS))
+					snprintf(buf, sizeof(buf), _(txt_post_newsgroups), BlankIfNull(tinrc.default_post_newsgroups));
+					if (!prompt_string_ptr_default(buf, &tinrc.default_post_newsgroups, _(txt_no_newsgroups), HIST_POST_NEWSGROUPS))
+						break;
+					str_trim(tinrc.default_post_newsgroups);
+					if (!*tinrc.default_post_newsgroups)
 						break;
 					if (group_find(tinrc.default_post_newsgroups, FALSE) == NULL) {
 						error_message(2, _(txt_not_in_active_file), tinrc.default_post_newsgroups);
 						break;
 					} else {
-						strcpy(buf, tinrc.default_post_newsgroups);
+						STRCPY(buf, tinrc.default_post_newsgroups);
 #if 1 /* TODO: fix the rest of the code so we don't need this anymore */
 						/*
 						 * this is a gross hack to avoid a crash in the
-						 * CHARSET_CONVERSION conversion case in new_part()
+						 * CHARSET_CONVERSION case in new_part()
 						 * which currently relies on CURR_GROUP
 						 */
 						selmenu.curr = my_group_add(buf, FALSE);
@@ -515,7 +551,7 @@ selection_page(
 					}
 				} else
 					STRCPY(buf, CURR_GROUP.name);
-				if (!can_post && !CURR_GROUP.bogus && !CURR_GROUP.attribute->mailing_list) {
+				if (!can_post && !CURR_GROUP.bogus && !(CURR_GROUP.attribute->mailing_list && *CURR_GROUP.attribute->mailing_list)) {
 					info_message(_(txt_cannot_post));
 					break;
 				}
@@ -532,7 +568,7 @@ selection_page(
 				break;
 
 			case GLOBAL_DISPLAY_POST_HISTORY:	/* display messages posted by user */
-				if (user_posted_messages())
+				if (post_hist_page())
 					show_selection_page();
 				break;
 
@@ -561,11 +597,18 @@ selection_page(
 						buf[j++] = ' ';
 					buf[j] = '\0';
 				}
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+				if ((wtmp = char2wchar_t(buf))) {
+					mark_screen(selmenu.curr, ucnt_offset, wtmp);
+					free(wtmp);
+				}
+#else
 				mark_screen(selmenu.curr, ucnt_offset, buf);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				break;
 
 			default:
-				info_message(_(txt_bad_command), printascii(key, func_to_key(GLOBAL_HELP, select_keys)));
+				info_message(_(txt_bad_command), PrintFuncKey(key, GLOBAL_HELP, select_keys));
 		}
 	}
 }
@@ -575,66 +618,102 @@ void
 show_selection_page(
 	void)
 {
-	char buf[LEN];
-	int i;
+	char buf[LEN], keyhelp[MAXKEYLEN];
+	char *title;
+	int i, keyhelplen;
 	size_t len;
+	const char *secflag = "";
+	t_bool recalc = !sel_fmt.str || !groupname_len || (need_parse_fmt & SELECT_LEVEL);
 
 	signal_context = cSelect;
 	currmenu = &selmenu;
-	parse_format_string(tinrc.select_format, &sel_fmt);
-	groupname_len = 0;
-	flags_offset = 0;
-	mark_offset = 0;
-	ucnt_offset = 0;
+
+	if (recalc) {
+		parse_format_string(tinrc.select_format, &sel_fmt);
+		need_parse_fmt &= ~SELECT_LEVEL;
+		groupname_len = 0;
+		flags_offset = 0;
+		mark_offset = 0;
+		ucnt_offset = 0;
+	}
+
+	if (use_nntps) {
+		if (insecure_nntps)
+			secflag = _(txt_selection_flag_insecure);
+		else
+			secflag = _(txt_selection_flag_secure);
+	}
 
 	if (read_news_via_nntp)
-		snprintf(buf, sizeof(buf), "%s (%s  %d%s)", _(txt_group_selection), nntp_server, selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
+		snprintf(buf, sizeof(buf), "%s (%s%s  %d%s)", _(txt_group_selection), nntp_server, secflag, selmenu.max, (tinrc.show_only_unread_groups ? _(txt_selection_flag_only_unread) : ""));
 	else
-		snprintf(buf, sizeof(buf), "%s (%d%s)", _(txt_group_selection), selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
+		snprintf(buf, sizeof(buf), "%s (%d%s)", _(txt_group_selection), selmenu.max, (tinrc.show_only_unread_groups ? _(txt_selection_flag_only_unread) : ""));
 
 	if (selmenu.curr < 0)
 		selmenu.curr = 0;
 
 	ClearScreen();
 	set_first_screen_item();
-	show_title(buf);
 
-	if (sel_fmt.len_grpname_max && !sel_fmt.len_grpname) {
-		/*
-		 * calculate max length of groupname field
-		 * if yanked in (yanked_out == FALSE) check all groups in active file
-		 * otherwise just subscribed to groups
-		 */
-		if (yanked_out) {
-			for (i = 0; i < selmenu.max; i++) {
-				if ((len = strwidth(active[my_group[i]].name)) > sel_fmt.len_grpname)
-					sel_fmt.len_grpname = len;
-			}
-		} else {
-			for_each_group(i) {
-				if ((len = strwidth(active[i].name)) > sel_fmt.len_grpname)
-					sel_fmt.len_grpname = len;
+	/*
+	 * determine max. length for centered title
+	 */
+	if (tinrc.show_help_mail_sign != SHOW_SIGN_NONE) {
+		if (tinrc.show_help_mail_sign == SHOW_SIGN_MAIL)
+			len = cCOLS - (2 * strwidth(_(txt_you_have_mail))) - 2;
+		else {
+			PrintFuncKey(keyhelp, GLOBAL_HELP, select_keys);
+			keyhelplen = strwidth(keyhelp);
+			if (tinrc.show_help_mail_sign == SHOW_SIGN_HELP)
+				len = cCOLS - (2 * (strwidth(_(txt_type_h_for_help)) - 2 + keyhelplen)) - 2;
+			else
+				len = cCOLS - (2 * MAX(strwidth(_(txt_type_h_for_help)) - 2 + keyhelplen, strwidth(_(txt_you_have_mail)))) - 2;
+		}
+	} else
+		len = cCOLS - 2;
+
+	title = strunc(buf, len);
+	show_title(title);
+	free(title);
+
+	if (recalc) {
+		if (sel_fmt.len_grpname_max && !sel_fmt.len_grpname) {
+			/*
+			* calculate max length of groupname field
+			* if yanked in (yanked_out == FALSE) check all groups in active file
+			* otherwise just subscribed to groups
+			*/
+			if (yanked_out) {
+				for (i = 0; i < selmenu.max; i++) {
+					if ((len = (size_t) strwidth(active[my_group[i]].name)) > sel_fmt.len_grpname)
+						sel_fmt.len_grpname = len;
+				}
+			} else {
+				for_each_group(i) {
+					if ((len = (size_t) strwidth(active[i].name)) > sel_fmt.len_grpname)
+						sel_fmt.len_grpname = len;
+				}
 			}
 		}
+
+		groupname_len = (sel_fmt.show_grpdesc && show_description) ? (int) sel_fmt.len_grpname_dsc : (int) sel_fmt.len_grpname;
+
+		if (groupname_len > (int) sel_fmt.len_grpname_max)
+			groupname_len = (int) sel_fmt.len_grpname_max;
+		if (groupname_len < 0)
+			groupname_len = 0;
+
+		if (!sel_fmt.len_grpdesc)
+			sel_fmt.len_grpdesc = (sel_fmt.len_grpname_max - (size_t) groupname_len);
+		else {
+			if (sel_fmt.len_grpdesc > (sel_fmt.len_grpname_max - (size_t) groupname_len))
+				sel_fmt.len_grpdesc = (sel_fmt.len_grpname_max - (size_t) groupname_len);
+		}
+
+		flags_offset = (int) (sel_fmt.flags_offset + (size_t) (sel_fmt.g_before_f ? groupname_len : 0) + (sel_fmt.d_before_f ? sel_fmt.len_grpdesc : 0));
+		ucnt_offset = (int) (sel_fmt.ucnt_offset + (size_t) (sel_fmt.g_before_u ? groupname_len : 0) + (sel_fmt.d_before_u ? sel_fmt.len_grpdesc : 0));
 	}
 
-	groupname_len = (sel_fmt.show_grpdesc && show_description) ? (int) sel_fmt.len_grpname_dsc : (int) sel_fmt.len_grpname;
-
-	if (groupname_len > (int) sel_fmt.len_grpname_max)
-		groupname_len = sel_fmt.len_grpname_max;
-	if (groupname_len < 0)
-		groupname_len = 0;
-
-	if (!sel_fmt.len_grpdesc)
-		sel_fmt.len_grpdesc = sel_fmt.len_grpname_max - groupname_len;
-	else {
-		if (sel_fmt.len_grpdesc > sel_fmt.len_grpname_max - groupname_len)
-			sel_fmt.len_grpdesc = sel_fmt.len_grpname_max - groupname_len;
-	}
-
-	flags_offset = sel_fmt.flags_offset + (sel_fmt.g_before_f ? groupname_len : 0) + (sel_fmt.d_before_f ? sel_fmt.len_grpdesc : 0);
-	ucnt_offset = sel_fmt.ucnt_offset + (sel_fmt.g_before_u ? groupname_len : 0) + (sel_fmt.d_before_u ? sel_fmt.len_grpdesc : 0);
-
 	for (i = selmenu.first; i < selmenu.first + NOTESLINES && i < selmenu.max; i++)
 		build_gline(i);
 
@@ -664,7 +743,7 @@ build_gline(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	char *name_buf = NULL;
 	char *desc_buf = NULL;
-	wchar_t *active_name, *active_name2, *active_desc, *active_desc2;
+	wchar_t *active_name, *active_name2 = NULL, *active_desc, *active_desc2;
 #else
 	char *active_name, *active_name2;
 	size_t fill, len_start;
@@ -676,15 +755,15 @@ build_gline(
 	 * make it the same size like in !USE_CURSES case to simplify the code
 	 */
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		sptr = my_malloc(cCOLS * MB_CUR_MAX + 2);
+		sptr = my_malloc((size_t) cCOLS * MB_CUR_MAX + 2);
 #	else
-		sptr = my_malloc(cCOLS + 2);
+		sptr = my_malloc((size_t) cCOLS + 2);
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #else
 	sptr = screen[INDEX2SNUM(i)].col;
 #endif /* USE_CURSES */
 
-	sptr[0] = '\0';
+	*sptr = '\0';
 	fmt = sel_fmt.str;
 	n = my_group[i];
 
@@ -707,13 +786,12 @@ build_gline(
 			case 'd':
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				if (show_description && active[n].description) {
-					active_desc = char2wchar_t(active[n].description);
-					if (active_desc) {
+					if ((active_desc = char2wchar_t(active[n].description)) != NULL) {
 						if ((active_desc2 = wcspart(active_desc, sel_fmt.len_grpdesc, TRUE)) != NULL) {
 							desc_buf = wchar_t2char(active_desc2);
-							free(active_desc);
 							free(active_desc2);
 						}
+						free(active_desc);
 					}
 					if (desc_buf) {
 						strcat(sptr, desc_buf);
@@ -742,7 +820,7 @@ build_gline(
 			case 'f':
 				/*
 				 * Display a flag for this group if needed
-				 * . Bogus groups are dumped immediately
+				 * . Bogus groups are dumped immediately 'D'
 				 * . Normal subscribed groups may be
 				 *   ' ' normal, 'X' not postable, 'M' moderated, '=' renamed
 				 * . Newgroups are 'N'
@@ -769,14 +847,17 @@ build_gline(
 				if (active_name && tinrc.abbreviate_groupname) {
 					active_name2 = abbr_wcsgroupname(active_name, groupname_len);
 					free(active_name);
-				} else
+				} else {
+					FreeIfNeeded(active_name2);
 					active_name2 = active_name;
+				}
 
 				if (active_name2 && (active_name = wcspart(active_name2, groupname_len, TRUE)) != NULL) {
-					free(active_name2);
 					name_buf = wchar_t2char(active_name);
 					free(active_name);
 				}
+				FreeAndNull(active_name2);
+
 				if (name_buf) {
 					strcat(sptr, name_buf);
 					FreeAndNull(name_buf);
@@ -801,11 +882,22 @@ build_gline(
 
 			case 'U':
 				if (active[my_group[i]].inrange) {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+					char tmp_buf[10];
+
+					buf = sptr + strlen(sptr);
+					for (j = 1; j <= sel_fmt.len_ucnt - (art_mark_width - (art_mark_width - wcwidth(tinrc.art_marked_inrange))); ++j)
+						*buf++ = ' ';
+					snprintf(tmp_buf, sizeof(tmp_buf), "%"T_CHAR_FMT, (wint_t) tinrc.art_marked_inrange);
+					*buf-- = '\0';
+					strcat(buf, tmp_buf);
+#else
 					buf = sptr + strlen(sptr);
 					for (j = 1; j < sel_fmt.len_ucnt; ++j)
 						*buf++ = ' ';
 					*buf++ = tinrc.art_marked_inrange;
 					*buf = '\0';
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				} else if (active[my_group[i]].newsrc.num_unread) {
 					int getart_limit;
 					t_artnum num_unread;
@@ -877,6 +969,8 @@ yank_active_file(
 		return;
 	}
 
+	need_parse_fmt |= SELECT_LEVEL;
+
 	if (yanked_out) {						/* Yank in */
 		int i;
 		int prevmax = selmenu.max;
@@ -893,7 +987,7 @@ yank_active_file(
 		selmenu.curr = save_restore_curr_group(FALSE);	/* Restore previous group position */
 		yanked_out = bool_not(yanked_out);
 		show_selection_page();
-		info_message(_(txt_yanked_groups), selmenu.max-prevmax, PLURAL(selmenu.max-prevmax, txt_group));
+		info_message(_(txt_yanked_groups), selmenu.max - prevmax, PLURAL(selmenu.max - prevmax, txt_group));
 	} else {							/* Yank out */
 		toggle_my_groups(NULL);
 		HpGlitch(erase_arrow());
@@ -936,8 +1030,7 @@ save_restore_curr_group(
 	 * Take a copy of the current groupname, if present
 	 */
 	if (saving) {
-		oldmax = selmenu.max;
-		if (oldmax)
+		if ((oldmax = selmenu.max))
 			oldgroup = my_strdup(CURR_GROUP.name);
 		return 0;
 	}
@@ -984,9 +1077,9 @@ choose_new_group(
 	char *prompt;
 	int idx;
 
-	prompt = fmt_string(_(txt_newsgroup), tinrc.default_goto_group);
+	prompt = fmt_string(_(txt_newsgroup), BlankIfNull(tinrc.default_goto_group));
 
-	if (!(prompt_string_default(prompt, tinrc.default_goto_group, "", HIST_GOTO_GROUP))) {
+	if (!(prompt_string_ptr_default(prompt, &tinrc.default_goto_group, "", HIST_GOTO_GROUP))) {
 		free(prompt);
 		return -1;
 	}
@@ -994,7 +1087,7 @@ choose_new_group(
 
 	str_trim(tinrc.default_goto_group);
 
-	if (tinrc.default_goto_group[0] == '\0')
+	if (!*tinrc.default_goto_group)
 		return -1;
 
 	clear_message();
@@ -1018,7 +1111,7 @@ skip_newgroups(
 
 	if (selmenu.max) {
 		while (i < selmenu.max && active[my_group[i]].newgroup)
-			i++;
+			++i;
 	}
 
 	return i;
@@ -1075,9 +1168,10 @@ reposition_group(
 	if (!prompt_string(buf, pos, HIST_MOVE_GROUP))
 		return default_num;
 
-	if (strlen(pos))
-		pos_num = ((pos[0] == '$') ? selmenu.max : atoi(pos));
-	else {
+	if (*pos) {
+		if ((pos_num = ((pos[0] == '$') ? selmenu.max : s2i(pos, 1, selmenu.max))) == default_num)
+			return default_num;
+	} else {
 		if (tinrc.default_move_group)
 			pos_num = tinrc.default_move_group;
 		else
@@ -1128,6 +1222,9 @@ catchup_group(
 	t_bool goto_next_unread_group)
 {
 	char *smsg = NULL;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wtmp;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	if ((!TINRC_CONFIRM_ACTION) || prompt_yn(sized_message(&smsg, _(txt_mark_group_read), group->name), TRUE) == 1) {
 		grp_mark_read(group, NULL);
@@ -1138,7 +1235,14 @@ catchup_group(
 			while (i < sel_fmt.len_ucnt)
 				buf[i++] = ' ';
 			buf[i] = '\0';
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			if ((wtmp = char2wchar_t(buf))) {
+				mark_screen(selmenu.curr, ucnt_offset, wtmp);
+				free(wtmp);
+			}
+#else
 			mark_screen(selmenu.curr, ucnt_offset, buf);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		}
 
 		if (goto_next_unread_group)
@@ -1299,7 +1403,7 @@ toggle_my_groups(
 	}
 #else
 	/* preserve group ordering based on newsrc */
-	if ((fp = fopen(newsrc, "r")) == NULL)
+	if ((fp = tin_fopen(newsrc, "r")) == NULL)
 		return;
 
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
@@ -1335,6 +1439,8 @@ subscribe_pattern(
 {
 	char buf[LEN];
 	int i, subscribe_num = 0;
+	size_t groups_size = 100;
+	struct t_group **groups;
 
 	if (!num_active || no_write)
 		return;
@@ -1344,23 +1450,33 @@ subscribe_pattern(
 		return;
 	}
 
+	groups = my_malloc(groups_size * sizeof(struct t_group *));
+
 	wait_message(0, "%s", message);
 
 	for_each_group(i) {
 		if (match_group_list(active[i].name, buf)) {
 			if (active[i].subscribed != (state != FALSE)) {
 				spin_cursor();
+				if ((size_t) subscribe_num == groups_size) {
+					groups_size <<= 1;
+					groups = my_realloc(groups, groups_size * sizeof(struct t_group *));
+				}
+				groups[subscribe_num] = &active[i];
 				/* If found and group is not subscribed add it to end of my_group[]. */
-				subscribe(&active[i], SUB_CHAR(state), TRUE);
 				if (state) {
 					my_group_add(active[i].name, FALSE);
 					grp_mark_unread(&active[i]);
 				}
-				subscribe_num++;
+				++subscribe_num;
 			}
 		}
 	}
 
+	bulk_subscribe(groups, subscribe_num, SUB_CHAR(state), TRUE);
+
+	free(groups);
+
 	if (subscribe_num) {
 		toggle_my_groups(NULL);
 		show_selection_page();
@@ -1373,12 +1489,13 @@ subscribe_pattern(
 /*
  * Does NOT return
  */
-static void
+_Noreturn static void
 select_quit(
 	void)
 {
 	write_config_file(local_config_file);
 	ClearScreen();
+	FreeAndNull(sel_fmt.str);
 	tin_done(EXIT_SUCCESS, NULL);	/* Tin END */
 }
 
@@ -1401,7 +1518,7 @@ static void
 select_read_group(
 	void)
 {
-	struct t_group *currgrp;
+	static const struct t_group *currgrp;
 
 	if (!selmenu.max || selmenu.curr == -1) {
 		info_message(_(txt_no_groups));
@@ -1428,7 +1545,7 @@ select_read_group(
  */
 static char *
 lookup_msgid(
-	char *msgid)
+	const char *msgid)
 {
 	if (read_news_via_nntp && !read_saved_news) {
 		if (!nntp_caps.hdr_cmd && !nntp_caps.xpat) {
@@ -1437,7 +1554,7 @@ lookup_msgid(
 		}
 		if (msgid) {
 			char *ptr, *r = NULL;
-			static char *x = NULL;
+			static char *x;
 			char buf[NNTP_STRLEN];
 			int ret;
 
@@ -1448,6 +1565,7 @@ lookup_msgid(
 				switch (ret) {
 					case OK_HEAD:
 					case OK_HDR:
+						x = NULL;
 						while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
 #		ifdef DEBUG
 							if (debug & DEBUG_NNTP)
@@ -1460,9 +1578,9 @@ lookup_msgid(
 								} else { /* DNEWS ("%d %s", num, grp) */
 									r = ptr;
 									while (*r && *r != ' ' && *r != '\t')
-										r++;
+										++r;
 									while (*r && (*r == ' ' || *r == '\t'))
-										r++;
+										++r;
 								}
 							}
 
@@ -1471,8 +1589,10 @@ lookup_msgid(
 									r = ptr + 2;
 							}
 
-							if (r)
+							if (r) {
+								FreeIfNeeded(x);	/* only required on bogus multi responses, just to be safe */
 								x = my_strdup(r);
+							}
 						}
 
 						if (x)
@@ -1504,9 +1624,9 @@ lookup_msgid(
 			}
 
 			if (nntp_caps.xpat) {
+				x = NULL;
 				snprintf(buf, sizeof(buf), "XPAT Newsgroups %s *", msgid);
 				ret = new_nntp_command(buf, OK_HEAD, NULL, 0);
-				r = NULL;
 				switch (ret) {
 					case OK_HEAD:
 						while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
@@ -1517,8 +1637,10 @@ lookup_msgid(
 							if (!strncmp(ptr, msgid, strlen(msgid)))
 								r = ptr + strlen(msgid) + 1;
 
-							if (r)
+							if (r) {
+								FreeIfNeeded(x); /* only required on bogus multi responses, just to be safe */
 								x = my_strdup(r);
+							}
 						}
 
 						if (x)
@@ -1558,49 +1680,70 @@ lookup_msgid(
  * If no group from the Newsgroups:-header is available, display the
  * contents of the header.
  */
-static int
+int
 show_article_by_msgid(
-	void)
+	const char *messageid)
 {
-	char id[LEN];
-	char *idptr;
+	char id[NNTP_STRLEN];	/* still way too big; RFC 3977 3.6 & RFC 5536 3.1.3 limit Message-ID to max 250 octets */
+	char *idptr = NULL;
 	char *newsgroups = NULL;
+	char *ngcpy, *cg, *ng;
 	int i, ret = 0;
 	struct t_article *art;
 	struct t_group *group = NULL;
+	struct t_group *tmp_group = NULL;
 	struct t_msgid *msgid = NULL;
 	t_bool tmp_cache_overview_files;
 	t_bool tmp_show_only_unread_arts;
 
-	if (!(read_news_via_nntp && !read_saved_news)) {
-		info_message("%s %s", _(txt_lookup_func_not_available), _(txt_lookup_func_not_nntp));
-		return -1;
+	if (!(read_news_via_nntp && !read_saved_news))
+		return LOOKUP_UNAVAIL;
+
+	if (messageid) {
+		/* sizeof(id) - 2 to have space for '>' later on if necessary */
+		if (snprintf(id + 1, sizeof(id) - 2, "%s", messageid) > 0)
+			idptr = str_trim(id + 1);
+	} else {
+		if (prompt_string(_(txt_enter_message_id), id + 1, HIST_MESSAGE_ID) && id[1])
+			idptr = str_trim(id + 1);
 	}
 
-	if (prompt_string(_(txt_enter_message_id), id + 1, HIST_MESSAGE_ID) && id[1]) {
-		idptr = str_trim(id + 1);
-		if (id[1] != '<') {
-			id[0] = '<';
-			strcat(id, ">");
-			idptr = id;
+	if (idptr) {
+		if (*idptr != '<') {
+			*(--idptr) = '<';
+			strcat(idptr, ">");
 		}
 		newsgroups = lookup_msgid(idptr);
 	}
 
 	if (!newsgroups)
-		return -1;
+		return LOOKUP_ART_UNAVAIL;
 
-	if ((group = get_group_from_list(newsgroups)) == NULL) {
-		info_message(strchr(newsgroups, ',') ? _(txt_lookup_show_groups) : _(txt_lookup_show_group), newsgroups);
-		free(newsgroups);
-		return -1;
+	/* set selmenu.curr */
+	ng = ngcpy = my_strdup(newsgroups); /* take a copy for strtok */
+	while ((cg = strtok(ng, ",")) != NULL) {
+		if ((selmenu.curr = my_group_add(cg, FALSE)) != -1)
+			break;
+		ng = NULL;
 	}
 
+	if (selmenu.curr == -1 || !cg || (group = get_group_from_list(cg)) == NULL) {
+		if (!cmdline.msgid)
+			info_message(strchr(newsgroups, ',') ? _(txt_lookup_show_groups) : _(txt_lookup_show_group), newsgroups);
+		else /* -L cmd. */
+			wait_message(2, strchr(newsgroups, ',') ? _(txt_lookup_show_groups) : _(txt_lookup_show_group), newsgroups);
+		free(newsgroups);
+		free(ngcpy);
+		return LOOKUP_FAILED;
+	}
+	free(ngcpy);
+
+	if (curr_group)
+		tmp_group = curr_group;
 	curr_group = group;
 	num_of_tagged_arts = 0;
 	range_active = FALSE;
-	last_resp = -1;
-	this_resp = -1;
+	this_resp = last_resp = -1;
 	tmp_cache_overview_files = tinrc.cache_overview_files;
 	tinrc.cache_overview_files = FALSE;
 	tmp_show_only_unread_arts = curr_group->attribute->show_only_unread_arts;
@@ -1613,32 +1756,29 @@ show_article_by_msgid(
 			FreeAndNull(art->msgid);
 		}
 		tin_errno = 0;
-		ret = -1;
+		ret = LOOKUP_FAILED;
 	}
 
 	if (!ret) {
 		grpmenu.first = 0;
 
-		if ((msgid = find_msgid(idptr)) == NULL) {
-			info_message(_(txt_art_unavailable));
-			ret = -1;
-		}
+		if (*idptr == '\0')
+			ret = LOOKUP_ART_UNAVAIL;
 
-		if (!ret && msgid->article == ART_UNAVAILABLE) {
-			info_message(_(txt_art_unavailable));
-			ret = -1;
-		}
+		if ((msgid = find_msgid(idptr)) == NULL)
+			ret = LOOKUP_ART_UNAVAIL;
 
-		if (!ret && which_thread(msgid->article) == -1) {
-			info_message(_(txt_no_last_message));
-			ret = -1;
-		}
+		if (!ret && msgid->article == ART_UNAVAILABLE)
+			ret = LOOKUP_ART_UNAVAIL;
+
+		if (!ret && which_thread(msgid->article) == -1)
+			ret = LOOKUP_NO_LAST;
 	}
 
 	if (!ret) {
-		switch ((i = show_page(group, msgid->article, NULL))) {
+		switch (show_page(group, msgid->article, NULL)) {
 			case GRP_QUIT:
-				ret = GRP_QUIT;
+				ret = LOOKUP_QUIT;
 				break;
 
 			default:
@@ -1649,8 +1789,23 @@ show_article_by_msgid(
 	free(newsgroups);
 	art_close(&pgart);
 	tinrc.cache_overview_files = tmp_cache_overview_files;
-	curr_group->attribute->show_only_unread_arts = tmp_show_only_unread_arts;
-	curr_group = NULL;
+	curr_group->attribute->show_only_unread_arts = CAST_BOOL(tmp_show_only_unread_arts);
+	if (tmp_group) {
+		curr_group = tmp_group;
+		if (!index_group(curr_group)) {
+			for_each_art(i) {
+				art = &arts[i];
+				FreeAndNull(art->refs);
+				FreeAndNull(art->msgid);
+			}
+			curr_group = NULL;
+			tin_errno = 0;
+			ret = LOOKUP_FAILED;
+		}
+	} else
+		curr_group = NULL;
+
+	this_resp = last_resp = -1;
 
 	return ret;
 }
@@ -1665,7 +1820,7 @@ get_group_from_list(
 {
 	char *ptr, *tr;
 	t_bool found = FALSE;
-	struct t_group *group = NULL;
+	struct t_group *group;
 
 	if (!newsgroups || (ptr = strtok(newsgroups, ",")) == NULL)
 		return NULL;
